package edu.udo.cs.ie.cowbots.arch.massim;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Vector;

import org.w3c.dom.Element;

import edu.udo.cs.ie.cowbots.arch.MassimAdapter;
import java.util.concurrent.locks.*;

/**
 * Implementation of the abstract class AbstractAgent,
 * wich represents an agent in the massim environment.
 * It is used to percieve data from massim and set the
 * next action performed by the agent.
 * The methods provided by the AbstractAgent are used
 * to connect and login to the server and to communicate
 * via xml documents.
 * 
 * @author dhoelzgen
 *
 */
public class MassimAgent extends AbstractAgent {
	
	/**
	 * The next action performed by the agent
	 */
	private String nextAction;
	
	/**
	 * Massim adapter, which can be called if new perceptions
	 * arrive.
	 */
	private MassimAdapter massimAdapter;
	
	/**
	 * Help vars, to store the relative agents position
	 */
	public int posX;
	public int posY;

	BufferedReader in;
	
	/**
	 * Empty constructor, without information about the massim server
	 */
	public MassimAgent(){
		this.nextAction = "skip";
		this.posX = 0;
		this.posY = 0;
	}

	/**
	 * Creates a new instance of a MassimAgent
	 * 
	 * @param massimAdapter The adapter which can be called if new perceptions arrive
	 * @param host The host massim is running on
	 * @param username The agents username
	 * @param pass The agents password
	 * @param port The port massim is listening to
	 */
	public MassimAgent(MassimAdapter massimAdapter, String host,String username, String pass,int port){
		this();
		this.setUsername(username);
		this.setPassword(pass);
		this.setHost(host);
		this.setPort(port);
		this.massimAdapter = massimAdapter;
			
		in = new BufferedReader(new InputStreamReader( System.in ) );
	}
	
	/**
	 * Sets the next action performed by the agent
	 * 
	 * @param action Action performed by the agent
	 * Possible actions are: skip, north, south, east, west,
	 * northwest, northeast, southwest, southeast
	 */
	public void setNextAction(String action) {
		//this.nextAction = action;
		super.sendResponseAction(action);
	}
	
	/**
	 * Processes xml formated new perceptions and sends the next
	 * action to the massim environment. This method is called
	 * by the abstract class.
	 */
	public void processRequestAction(Element perception, Element target,
		long currenttime, long deadline) {
		this.posX = Integer.parseInt(perception.getAttribute("posx"));
		this.posY = Integer.parseInt(perception.getAttribute("posy"));
		this.massimAdapter.processMassimPercepts(perception);
		
		// TODO Results in delay for one step, should be changed soon...
		String tmpAction = this.nextAction;
		this.nextAction = "skip";
		if (target != null)
			target.setAttribute("type", tmpAction);
	}
	
	/**
	 * Processes information about the end of a simulation. This method
	 * is called by the abstract class.
	 */
	public void processSimulationEnd(Element perception, long currenttime) {
		this.massimAdapter.processSimulationEnd(perception, currenttime);
	}
	
	/**
	 * Processes information about the start of a simulation. This method
	 * is called by the abstract class.
	 */
	public void processSimulationStart(Element perception, long currenttime) {
		this.massimAdapter.processMassimSimStart(perception);
	}
	
	/**
	 * Processes information about the log in process. This method
	 * is called by the abstract class.
	 */
	public void processLogIn() {
		
	}
}
