package edu.udo.cs.ie.cowbots.bdi.brf;

import java.util.*;
import jason.asSyntax.*;
import jason.asSemantics.*;
import jason.runtime.*;
import edu.udo.cs.ie.cowbots.logic.*;
import edu.udo.cs.ie.cowbots.logic.solver.*;
import edu.udo.cs.ie.cowbots.ui.*;
import org.w3c.dom.*;


/**
 * the basic interface for any class that represents a
 * revision function within the cowbot bdi framework. it 
 * controls the updates of an agent's epistemic state.
 *   
 * @author Thomas
 *
 */
public abstract class CRevisionComponent {
	
	/**
	 * this method is used to integrate percepts from
	 * the environment into the agent's belief base.
	 * 
	 * @param per	the list of environmental percepts
	 * @return		true if beliefs were changed, false otherwise
	 */
	public abstract boolean revise_percepts(List<Literal> per, List<ELPLiteral> explore);

	
	/**
	 * this method is used to handle belief addition and deletion
	 * from jason.
	 * 
	 *  @return list of added and deleted literals to fire jason events (can be null)
	 */
	public abstract List<Literal>[] revise_belief(Literal beliefToAdd, Literal beliefToDel,  Intention i);
	
	
	/**
	 * adds a logical program to an agents beliefs
	 * @param program	logical program to add
	 * @param source	source (agent name) of program 
	 */
	public abstract void addProgram(ELP program, String source);
	
	/**
	 * this method queues an incoming message. the deliberation may
	 * then decide to handle the message
	 */
	public abstract void queueMessage(Message m);
	
	/**
	 * this method should process all "tell" messages previously
	 * queued by queueMessage. 
	 * @return	list of added and deleted literals to fire jason events (can be null) 
	 */
	public abstract List<Literal>[] processMessages(String agName);
	
	
	/**
	 * returns the epistemic state of an agent 
	 * 
	 * @return 	epistemic state
	 */
	public abstract IEpistemicState getEpistemicState();
	
	
	/**
	 * initialization routine, allows the cowbot revision to be
	 * configured with user parameters. 
	 * 
	 * @param brfConfig		XML Node named "brf" containing setup data
	 * @param configName	user configuration override from mas2j
	 * @param dlv			instance of dlv asp solver
	 */
	public abstract void init(Settings stts, DLV dlv, ICowbotUI ui) throws Exception;
	
	
	/**
	 * returns a tupel of two lists describing changes in an
	 * agent's beliefs. this is used to update the jason belief base
	 * properly after the epistemic state was revised.
	 * 
	 * @return tupel <belAdd[], belDel[]> 
	 */
	public abstract List<Literal>[] getChangeOfBeliefs();
	
	
	/**
	 * only used internally, do not care about!
	 * @return true or false
	 */
	public abstract boolean newRevisionMode();
	
	/**
	 * this method adds a logical program to
	 * the program storage
	 * @param name
	 * @param program
	 */
	public abstract void addUserProgram(String name, ELP program);
	
	
	/**
	 * this method returns the logical belief base
	 * of an agent.
	 * 
	 * @return
	 */
	public abstract BeliefBase getBeliefBase();
	
	
	/**
	 * resets the revision component (is set back
	 * to a fresh initialized state).
	 */
	public abstract void reset();
}
