package edu.udo.cs.ie.cowbots.bdi.brf;

import java.util.*;

import jason.asSyntax.ListTerm;
import jason.asSyntax.Literal;
import jason.asSyntax.Rule;
import jason.asSyntax.Term;
import edu.udo.cs.ie.cowbots.logic.ELP;
import edu.udo.cs.ie.cowbots.logic.ELPAtom;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.logic.ELPRule;
import edu.udo.cs.ie.cowbots.logic.NegLiteral;

/**
 * This class translates jason literals into elps and backwards.
 * 
 * @author Thomas, Boguslaw, Fabian
 * 
 */

public class Parser {

	
	/**
	 * translates an annotated jason literal into an elp literal.
	 * 
	 * @param l				jason literal
	 * @param worldtime		time
	 * @return				elpliteral representation of l
	 */
	public static ELPLiteral parseLiteralFromPlan(Literal l, int worldtime, String sourceOverride)
	{
		// return value
		ELPLiteral ret = null;
		
		if (!l.hasSource())
			return Parser.parseJLiteral(l);
		
		// new atom
		ELPAtom atom = new ELPAtom(l.getFunctor(), l.getArity()+1);
		
		// copy terms (if any)
		// has source annot?
		
		if (l.hasSource()) {
			ListTerm lt = l.getSources();
			if (lt.size() > 1) {
				System.out.println("ms: "+lt);
			}
			String s = l.getSources().get(0).toString();
			
			atom.setTerm(0,l.getSources().get(0).toString());
		} else {
			atom.setTerm(0,sourceOverride);
			System.out.println("source missing: "+l);
			
		}
		for (int i = 0; i < l.getArity(); i++)
			atom.setTerm(i+1, l.getTerm(i).toString());
		//atom.setTerm(atom.getArity()-1, ""+worldtime);
		// copy sign
		ret = (l.negated())? new NegLiteral(atom) : atom;
		
		// done
		return ret;
	}
	
	public static Literal parsePlanLiteralToJason(Literal l) {
		
		// translate back
		Term tm[] = l.getTermsArray();
		String pr = l.getFunctor().toString();
		if (tm.length > 1) {
			pr += "(";
			
			for (int ii = 1; ii <= tm.length - 1; ii++) {
				if (ii > 1)
					pr+=",";
				pr += tm[ii].toString();
			}
			pr += ")";
		}

		Literal ll = Literal.parseLiteral(pr);
		ll.addSource(l.getTerm(0));
		
		return ll;
	}
	
	
	/**
	 * converts an elp literal to a jason literal, removes
	 * first term from predicate and adds it as source
	 * 
	 * @param l
	 * @return
	 */
	public static Literal toJasonWithSourceTerm(ELPLiteral l) {
		
		// translate back
		ELPAtom atm = l.getAtom();
		String pr = null;
		if (l.isStrictNegated())
			pr = "~";
		else
			pr = "";
		pr += atm.getPredicate();
		if (atm.getArity() > 1) {
			pr += "(";
			
			for (int ii = 1; ii <= atm.getArity() - 1; ii++) {
				if (ii > 1)
					pr+=",";
				pr += atm.getTerm(ii);
			}
			pr += ")";
		}

		Literal ll = Literal.parseLiteral(pr);
		ll.addSource( Literal.parse(atm.getTerm(0)) );
		
		return ll;
	}
	
	
	/**
	 * converts an elp literal to a jason literal, and
	 * additionally sets the first term as a source
	 * annotation (but does not remove first term).
	 * 
	 * @param l
	 * @return
	 */	
	public static Literal toJasonWithSourceRewrite(ELPLiteral l) {
		Literal ret = Literal.parseLiteral(l.toString());
		if (l.getAtom().getArity() > 0)
		ret.addSource( Literal.parse(l.getAtom().getTerm(0)));
		return ret;
	}
	
	
	/**
	 * this method converts jason literals into elp rules.
	 * 
	 * @param l		jason literals
	 * @return		list of elp rules
	 */
	public static ArrayList<ELPRule> parseJLiteralToRules(Literal l) {
		ArrayList<ELPRule> tempRules = new ArrayList<ELPRule>();

		String literalToParse;

			
		literalToParse = l.toString();
		// TODO trim free spaces between chars
		literalToParse.trim();
		// Is this literal a rule ?
		if (l.isRule()) {
			Rule jRule = (Rule) l;
			ELPRule elpRule = new ELPRule();

			// Is the head of the rule negated?
			if (jRule.negated()) {
				elpRule.addHead(new NegLiteral(new ELPAtom(jRule
						.toStringAsTerm())));
			}
			// The head of the rule is positive
			else {
				elpRule.addHead(new ELPAtom(jRule.toStringAsTerm()));
			}
			// TODO add logical and arithmetical expressions, for now play only
			// with literals
			elpRule.addBody(new ELPAtom(jRule.getBody().toString()));

			tempRules.add(elpRule);

		}// isFact
		else {
			String[] termList = new String[l.getArity()];
			ELPRule r = new ELPRule();
			for (int i = 0; i < l.getArity(); i++) {
				termList[i] = l.getTerm(i).toString();
			}
			ELPAtom atom = new ELPAtom(l.getFunctor(), termList);
			// If literal is negated then create NegLiteral
			if (l.negated()) {
				NegLiteral negLit = new NegLiteral(atom);
				r.addHead(negLit);
			}
			// If not negated create positive atom
			else {
				r.addHead(atom);
			}
			tempRules.add(r);
		}

		return tempRules;
	}
	
	/**
	 * translates a jason literal to an ELPLiteral
	 * 
	 * @param l		jason literal to translate
	 * @return		an elp-literal (positive or strict negated)
	 */
	public static ELPLiteral parseJLiteral(Literal l) {
		// return value
		ELPLiteral ret = null;
		
		// new atom
		ELPAtom atom = new ELPAtom(l.getFunctor(), l.getArity());
		
		// copy terms (if any)
		for (int i = 0; i < l.getArity(); i++)
			atom.setTerm(i, l.getTerm(i).toString());
		
		// copy sign
		ret = (l.negated())? new NegLiteral(atom) : atom;
		
		// done
		return ret;
	}
	
	/**
	 * converts a collection of elp literals into a list of
	 * jason literals
	 * 
	 * @param lits	collection of ELPLiterals
	 * @return	list of jason literals
	 */
	public static List<Literal> toJasonLiterals(Collection<ELPLiteral> lits) {
		ArrayList<Literal> ret = new ArrayList<Literal>();
		
		for( ELPLiteral l : lits) {
			ret.add( Literal.parseLiteral(l.toString()));
		}
		
		return ret;
	}
}
