package edu.udo.cs.ie.cowbots.bdi.brf.operators;

import java.util.*;

import edu.udo.cs.ie.cowbots.bdi.brf.IInferenceOperator;
import edu.udo.cs.ie.cowbots.bdi.brf.InformationObject;
import edu.udo.cs.ie.cowbots.logic.ELP;
import edu.udo.cs.ie.cowbots.logic.ELPAtom;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.logic.ELPRule;
import edu.udo.cs.ie.cowbots.logic.NegLiteral;
import edu.udo.cs.ie.cowbots.logic.NotLiteral;
import edu.udo.cs.ie.cowbots.logic.solver.AnswerSet;
import edu.udo.cs.ie.cowbots.logic.solver.DLV;

/**
 * a believe operator based on the ideas behind
 * causal rejection, with constraint handling
 * based upon revision programming.
 *  
 * @author Thomas
 *
 */
public class SURI implements IInferenceOperator {

	
	Map<String, List<ELPLiteral>> rejSet;
	Set<String> savePreds = new HashSet<String>();
	
	@Override
	public ELP createBeliefState(List<InformationObject> ioList) {
		
		// some internally used variables
		Set<ELPLiteral> heads = new HashSet<ELPLiteral>();
		List<ELPRule> constraints = new LinkedList<ELPRule>();
		Set<String> littypes = new HashSet<String>();
		
		// resulting update program
		ELP updSeq = new ELP();
		ELPRule r = new ELPRule();
		int counter = 0;
		
		// create rejection rules
		for (InformationObject io : ioList) {
			ELP prog = io.getELP();
			for (ELPRule rule : prog.getRules()) {
			
				++counter;
				
				if (rule.nHead() == 0) {
					constraints.add(rule);
					continue;
				}
				
				// rewrite head
				ELPLiteral head = rule.getHead(0);
				
				// support save predicates
				if (savePreds.contains(head.getAtom().getPredicate())) {
					r = new ELPRule();
					r.addHead(head);
					updSeq.addRule(r);
					
					// no counter increasment required for save rules,
					// this limits the integer range
					--counter;
				} else {
				
					heads.add( head );
					String lt = (head.isStrictNegated())?"n":"p";
					lt += "!" + head.getAtom().getPredicate();
					lt += "!" + head.getAtom().getArity();
					littypes.add(lt);
					
					// make a rule from r
					// create head
					r = new ELPRule();
					r.addHead( indexLiteral( head, ""+counter ));
									
					// create body
					for (int i = 0; i < rule.nBody(); i++)
						r.addBody( rule.getBody(i));
					
					/*
					if (rule.nBody()>0) {
						//r.addBody( rejLiteral(inverse(head)));
						r.addBody( new NotLiteral( inverse(head)));
					}
					*/
					
					updSeq.addRule(r);
				}
			}
		}
		
		// process constraints
		// reinterprets any constraint ":- C0, C1..Cn as
		// rej(C0) :- C1..Cn
		// this allows processing constraints as out-rules
		for (ELPRule c : constraints) {
			
			// every constraint c with body literals A, B1 .. Bn,
			// and safe predicates P(c) (arithmetic expression, etc)
			// 		:- A, B1..Bn, P(c)
			// is converted to
			// 	rej(A) :- index(B1), not rej(B1), ..., index(Bn), not rej(Bn), P(c)		
			ELPLiteral rejL = rejLiteral( c.getBody(0));
			
			boolean useIndex = true;
			
			// sorry, no time to rewrite blanks, programs
			// should be written according to that weakness
			//int blankIndex = 0;
			
			r = new ELPRule();
			r.addHead(rejL);
			for (int i = 0; i < c.nBody(); i++) {
				ELPLiteral l = c.getBody(i);
				if (savePreds.contains(l.getAtom().getPredicate())) {
					// save predicate
					r.addBody(l);
				} else if (l.isArithmetic()) {
					// arithmetic or special
					r.addBody( l );
					useIndex = false;
				} else if (l.isDefaultNegated()) {
					// add default negated
					r.addBody(l);
				} else {
					// not default negated, add indexed version
					r.addBody( indexLiteral(l,"SR"+i));
					if (i>0)
						r.addBody( new NotLiteral( rejLiteral(l) ));				
				}
			}
			
			// care about literal order
			if (useIndex && (r.nBody()>1)) {
				// get second literal
				r.addBody( ELPAtom.Less("SR0", "SR1"));
			}
			
			updSeq.addRule(r);
		}
		
		// chain literals (like in causal rejection approaches).
		for (String lt : littypes) {
			
			// any literal is encoded as:
			// SIGN!PREDICATE_SYMBOL!ARITY
			String decode[] = lt.split("!",3);
			
			int ari = Integer.parseInt(decode[2]);			
			ELPAtom la = new ELPAtom(decode[1],ari);
			ELPLiteral l = la;
			if (decode[0].charAt(0) == 'n')
				l = new NegLiteral(la);
			for (int i = 0; i < ari; i++) {
				l.getAtom().setTerm(i, "Va"+i);
			}
			
			// create rejection literal
			ELPLiteral rejL = rejLiteral(l);
			
			// propagation rule:
			// a :- index(a,i), not rej(a)
			r = new ELPRule();
			r.addHead(l);
			r.addBody( indexLiteral(l,"Idx"));
			r.addBody( new NotLiteral( rejL ));
			updSeq.addRule(r);
			
			// chain rule: higher
			// higher(a,I) :- index(a,I), index(a,IPrime), I < IPrime
			r = new ELPRule();
			r.addHead( higher(l,"Idx"));
			r.addBody( indexLiteral(l,"Idx"));
			r.addBody( indexLiteral(l,"Idx2"));
			r.addBody( ELPAtom.Gtr("Idx2", "Idx"));
			updSeq.addRule(r);
			
			// chain rule: max
			// max(a,I) :- index(a,I), not higher(a,I)
			r = new ELPRule();
			r.addHead( max(l,"Idx") );
			r.addBody( indexLiteral(l, "Idx"));
			r.addBody( new NotLiteral( higher(l,"Idx") ));
			updSeq.addRule(r);
			
			// reject rules
			// rej(a) :- max(a,Idx), max(~a, Idx2), Idx<Idx2, not rej(~a)
			r = new ELPRule();
			r.addHead( rejL );
			r.addBody( max(l,"Idx"));
			r.addBody( max(inverse(l),"Idx2"));
			r.addBody( ELPAtom.Less("Idx", "Idx2"));
			r.addBody( new NotLiteral( rejLiteral(inverse(l)) ));
			updSeq.addRule(r);
		}
		
		return updSeq;
	}
	
	/**
	 * auxiliary function that creates an indexed (or priorized)
	 * version of a literal.
	 * @param l			atom with sign S and any predicate N, terms T 
	 * @param p			priority 1
	 * @param pPrime	priority 2
	 * @return			rejindex_S_N(T, p, pPrime)
	 */
	protected ELPLiteral indexLiteral(ELPLiteral l, String counter) {
		String name = "index_p_";
		if (l.isStrictNegated())
			name = "index_n_";
		name += l.getAtom().getPredicate();
		ELPAtom atm = new ELPAtom( name, l.getAtom().getArity()+1 );
		for (int i = 0; i < l.getAtom().getArity(); i++)
			atm.setTerm(i, l.getAtom().getTerm(i));
		atm.setTerm( atm.getArity()-1, counter);
		return atm;
	}
	
	/**
	 * auxiliary function that creates a reject literal for a literal 
	 * @param l			atom with sign S and any predicate N, terms T 
	 * @return			rej_S_N(T)
	 */
	protected ELPLiteral rejLiteral(ELPLiteral l) {
		String name = "rej_p_";
		if (l.isStrictNegated())
			name = "rej_n_";
		name += l.getAtom().getPredicate();
		ELPAtom atm = new ELPAtom( name, l.getAtom().getArity() );
		for (int i = 0; i < l.getAtom().getArity(); i++)
			atm.setTerm(i, l.getAtom().getTerm(i));
		return atm;
	}
	
	/**
	 * auxiliary function that create a priority comparison reject literal 
	 * @param l			atom with sign S and any predicate N, terms T 
	 * @param p			priority 1
	 * @param pPrime	priority 2
	 * @return			rejindex_S_N(T, p, pPrime)
	 */
	protected ELPLiteral rejLiteralIndex(ELPLiteral l, String p, String pPrime) {
		String name = "rejindex_p_";
		if (l.isStrictNegated())
			name = "rejindex_n_";
		name += l.getAtom().getPredicate();
		ELPAtom atm = new ELPAtom( name, l.getAtom().getArity()+2 );
		for (int i = 0; i < l.getAtom().getArity(); i++)
			atm.setTerm(i, l.getAtom().getTerm(i));
		atm.setTerm( atm.getArity()-2, p);
		atm.setTerm( atm.getArity()-1, pPrime);
		return atm;
	}
	
	/**
	 * returns the inverse of a literal
	 * 
	 * @param l	= (Not) ~L
	 * @return	(Not) L if l = (Not) ~L (and vice versa)
	 */
	protected ELPLiteral inverse( ELPLiteral l ) {
		ELPLiteral ret;
		if (l.isStrictNegated())
			ret = l.getAtom();
		else
			ret = new NegLiteral( l.getAtom() );
		
		if (l.isDefaultNegated())
			ret = new NotLiteral( ret );
		
		return ret;
	}
	
	/**
	 * auxiliary function, creates a "higher_" literal from a literal.
	 * 
	 * @param l			literal l with sign S (p=+, n=~), functor N, terms T
	 * @param index		priority index
	 * @return			higher_S_N(T, index)
	 */
	protected ELPLiteral higher( ELPLiteral l, String index ) {
		String name = "higher_p_";
		if (l.isStrictNegated())
			name = "higher_n_";
		name += l.getAtom().getPredicate();
		ELPAtom atm = new ELPAtom( name, l.getAtom().getArity()+1 );
		for (int i = 0; i < l.getAtom().getArity(); i++)
			atm.setTerm(i, l.getAtom().getTerm(i));
		atm.setTerm( atm.getArity()-1, index);
		return atm;
	}
	
	/**
	 * auxiliary function, creates a max literal from a literal.
	 * 
	 * @param l			literal l with sign S (p=+, n=~), functor N, terms T
	 * @param index		priority index
	 * @return			max_S_N(T, index)
	 */	
	protected ELPLiteral max(ELPLiteral l, String index) {
		
		String name = "max_p_";
		if (l.isStrictNegated())
			name = "max_n_";
		name += l.getAtom().getPredicate();
		ELPAtom atm = new ELPAtom( name, l.getAtom().getArity()+1 );
		for (int i = 0; i < l.getAtom().getArity(); i++)
			atm.setTerm(i, l.getAtom().getTerm(i));
		atm.setTerm( atm.getArity()-1, index);		
		
		return atm;
	}

	@Override
	public AnswerSet selectAnswerSet(List<AnswerSet> asl, DLV dlv) {
		if (asl.size()<=0)
			return null;
		else {
			cleanAnswerSet(asl.get(0).literals);
			return asl.get(0);
		}
	}
	
	/**
	 * removes extended predicates from an answer set, also
	 * builds internal reject tables
	 * @param as
	 */
	protected void cleanAnswerSet(Set<ELPLiteral> as) {
		// reset reject set
		this.clearRejSet();
		
		// process elements
		Iterator<ELPLiteral> iter = as.iterator();
		while (iter.hasNext()) {
			ELPLiteral l = iter.next();
			String functor = l.getAtom().getPredicate();
			
			if (functor.startsWith("index_")) {
				iter.remove();
			}
			if (functor.startsWith("rej_")) {
				// strip out predicate
				boolean negated = (functor.charAt(4)=='p')? false:true;
				String predicate = functor.substring(6, functor.length());
				ELPAtom atm = new ELPAtom(predicate, l.getAtom().getArity());
				atm.setTerms(l.getAtom());
				ELPLiteral rejL = (negated)?new NegLiteral(atm):atm;
				this.add2RejSet(rejL);
				
				// and remove
				iter.remove();
			}
			if (functor.startsWith("max_")) {
				iter.remove();
			}
			if (functor.startsWith("higher_")) {
				iter.remove();
			}			
		}	
	}

	@Override
	public Map<String, List<ELPLiteral>> getRejectSet() {		
		return this.rejSet;
	}
	
	/**
	 * resets the reject set
	 */
	protected void clearRejSet() {
		this.rejSet = new HashMap<String, List<ELPLiteral>>();
	}
	
	/**
	 * inserts a literal into the reject set 
	 * @param l	literal to be inserted
	 */
	protected void add2RejSet(ELPLiteral l) {
		String functor = l.getAtom().getPredicate();
		if ( !rejSet.containsKey(functor) ) {
			rejSet.put(functor, new LinkedList<ELPLiteral>());
		}
		rejSet.get(functor).add(l);
	}

	@Override
	public void setSavePredicates(Set<String> savePreds) {
		this.savePreds = savePreds;
	}
}
