package edu.udo.cs.ie.cowbots.bdi.desgen;

import java.util.List;
import java.util.TreeSet;

import jason.asSyntax.Literal;
import edu.udo.cs.ie.cowbots.bdi.brf.IEpistemicState;
import edu.udo.cs.ie.cowbots.logic.ELP;

/**
 * Default implementation of a cowbots desire.
 * 
 * @author dhoelzgen
 *
 */
public class DefaultDesire extends CDesire {
	
	protected Literal uniqueName;
	protected Integer intensity;
	
	protected ELP updateELP;
	protected ELP mitigateELP;
	
	public DefaultDesire(Literal uniqueName, int i) {
		this.uniqueName = uniqueName;
		this.intensity = i;
	}
	
	public DefaultDesire(Literal uniqueName, int i, ELP updateELP, ELP mitigateELP) {
		this (uniqueName, i);
		this.updateELP = updateELP;
		this.mitigateELP = mitigateELP;
	}
	
	@Override
	public int getIntensity() {
		return intensity;
	}

	@Override
	public Literal getUniqueName() {
		return uniqueName;
	}

	@Override
	public int updateIntensity(IEpistemicState bel) {
		int newint =  this.getNewIntensityWithELP(bel, this.updateELP);
		if (newint != -1) this.intensity = newint;
		return this.intensity;
	}

	@Override
	public int updateMitigation(IEpistemicState bel) {
		int newint = this.getNewIntensityWithELP(bel, this.mitigateELP);
		if (newint != -1) this.intensity = newint;
		return this.intensity;
	}
	
	@Override
	public int compareTo(CDesire o) {
		return this.intensity.compareTo(o.getIntensity());
	}
	
	@Override
	public String toString() {
		return this.uniqueName.toString() + ", " + this.intensity.toString();
	}
	
	protected int getNewIntensityWithELP(IEpistemicState bel, ELP elp) {
		List<Literal> intensityList = bel.getBeliefs(elp);

		int rInt = -1;

		try {
			// Search for desirefunctionnewint(X), X is new intensity
			for (Literal lit : intensityList) {
				if (lit.toString().startsWith("desirefunctionnewint")) {
					rInt = Integer.parseInt(lit.getTerm(0).toString());
				}
			}
		} catch (Exception ex) {
			// TODO: Errorhandling
			System.out
					.println("Warning: Could not update intensity for desire "
							+ this.uniqueName.toString());
		}

		return rInt;
	}

}
