package edu.udo.cs.ie.cowbots.bdi.desgen;

import jason.asSyntax.Literal;

import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.bdi.brf.IEpistemicState;
import edu.udo.cs.ie.cowbots.logic.ELP;

/**
 * Default implementation of a cowbots motive.
 * 
 * @author dhoelzgen
 *
 */
public class DefaultMotive implements IMotive {

	private String uniqueName;
	
	private String generateELP;
	private String updateDesireELP;
	private String mitigateDesireELP;
	private String rawUniqueName;
	
	// TODO: Implement dynamic count of instance placeholders
	private final String INSTANCE_PLACEHOLDER = "$INSTANCE$";
	private final String INSTANCE_PLACEHOLDER2 = "$INSTANCETWO$";
	
	public DefaultMotive(String motiveName, String generateELP, String updateDesireELP, String mitigateDesireELP, String rawUniqueName) {
		this.uniqueName = motiveName;
		
		this.generateELP = generateELP;
		this.updateDesireELP = updateDesireELP;
		this.mitigateDesireELP = mitigateDesireELP;
		this.rawUniqueName = rawUniqueName;
	}
	
	@Override
	public List<CDesire> generateDesire(IEpistemicState bel) {
		// Get matching literals to the motives generate function
		ELP genELP = new ELP();
		genELP.addRaw(this.generateELP);
		List<Literal> newDesireList = bel.getBeliefs(genELP);
		
		List<CDesire> rDesireList = new LinkedList<CDesire>();
		
		// Search for motivegeneratorfound(X), instantiate desires with X
		for (Literal lit : newDesireList) {
			if (lit.toString().startsWith("motivegeneratorfound")) {
				
				String ivar = "";
				String ivar2 = "";
				
				if (lit.getTerms().size() == 1) {
					ivar = lit.getTerm(0).toString();
				} else if (lit.getTerms().size() == 1) {
					ivar = lit.getTerm(0).toString();
					ivar2 = lit.getTerm(1).toString();
				}
				
				// Instantiate unique name and ELPs with X
				String newUniqueName = this.rawUniqueName.replace(INSTANCE_PLACEHOLDER, ivar);
				newUniqueName = newUniqueName.replace(INSTANCE_PLACEHOLDER2, ivar2);
				Literal desire_uniqueName = Literal.parseLiteral(newUniqueName);
				
				String instUpdateELP = this.updateDesireELP;
				String instMitigateELP = this.mitigateDesireELP;
				
				instUpdateELP = instUpdateELP.replace(INSTANCE_PLACEHOLDER, ivar);
				instUpdateELP = instUpdateELP.replace(INSTANCE_PLACEHOLDER2, ivar2);
				
				instMitigateELP = instMitigateELP.replace(INSTANCE_PLACEHOLDER, ivar);
				instMitigateELP = instMitigateELP.replace(INSTANCE_PLACEHOLDER2, ivar2);
				
				// Use new Cowbot Desire, for old one ELPs are needed
				// TODO: Motive should choose the right class by config
				rDesireList.add(new CowbotDesire(desire_uniqueName, 0, 
						instUpdateELP, instMitigateELP));
			}
		}
		
		return rDesireList;
	}

	@Override
	public String getUniqueName() {
		return uniqueName;
	}

}
