package edu.udo.cs.ie.cowbots.elpParser.dlp;

import java.util.List;
import java.util.regex.Pattern;

import edu.udo.cs.ie.cowbots.logic.ELPAtom;



/**
 * this class extends ELPAtom
 * 
 * @author Wei
 * @see ELPAtom
 */
public class DLPAtom extends ELPAtom implements DLPLiteral {

	private Predicate predicate;
	private DLPTermList tl;	
	
    public DLPAtom(Predicate p, DLPTermList ts){
    	
    	super(p.getName(), ts.toArray());
    	this.predicate = p;
    	this.tl = ts;
   
    }
    
	/**
	 * default copy constructor
	 */
	public DLPAtom(DLPAtom other) {
		
		super(other);

	}
	
	/**
	 * returns the predicate symbol of this atom
	 * @return	predicate symbol {@link String}
	 */
	public String getPredicate() {
		return predicate.getName();
	}
	
	/**
	 * returns the arity of term
	 * @return	arity  {@link Integer}
	 */
	public int getTermArity() {
		
		int arity = 0;		
		List<DLPTerm> ts = this.tl.getAllTerms();
		
		for(DLPTerm term:ts){
			if(term instanceof DLPOperator)
				arity += 0;
			else 
				arity ++;
			
		}			
		
		return arity;
	}
	
	public int getArtiy(){
		
		return tl.getAllTerms().size();
		
	}
	
	/**
	 * returns a term at a given index
	 * @param i		term index i {@link Integer}
	 * @return		term i {@link Integer}
	 */
	public String getTerm(int i) {
		return tl.getTerm(i).getName();
	}
	
	
	public void setTerms(DLPAtom other) {
		if (this.getArity() != other.getArity())
			return;
		for(int i = 0; i < this.getArity(); i++)
			this.setTerm(i, other.getTerm(i));
	}
	
	
	public String[] getTerms() {
		return tl.toArray();
	}
	
	/**
	 * returns the predicate symbol and the arity.
	 * 
	 * @return	string composed of predicate name, followed by "/" and arity {@link String}
	 */
	public String getType() {
		return this.getPredicate() + "/" + this.getArity();
	}
	
	/**
	 * translates the atom into a human readable string.
	 * the output conforms datalog syntax 
	 */
	public String toString() {
		if (tl == null)
			return predicate.getName();
		else {
			String ret = predicate.getName() +"(";
			
			
			
			int size = tl.getListSize();
			
			if (size != 0)
				ret += tl.getTerm(0);
			
			for (int i = 1; i < size; i++) {
				ret += ","+tl.getTerm(i);
			}
			ret += ")";
			
			return ret;
		}
	}
	
	@Override
	public boolean isAtom() {
		return true;
	}

	@Override
	public boolean isDefaultNegated() {
		return false;
	}

	@Override
	public boolean isStrictNegated() {
		return false;
	}

	@Override
	public DLPLiteral getLiteral() {
		return this;
	}

	@Override
	public DLPAtom getAtom() {
		return this;
	}
	
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof DLPAtom))
			return false;
		
		DLPAtom a = (DLPAtom) o;
		
		return a.toString().equals(this.toString());
	}


	@Override
	public boolean isArithmetic() {
		return false;
	}

	@Override
	public boolean LitIsAfact(boolean b) {
		// TODO Auto-generated method stub
		return b;
	}

	


}



