package edu.udo.cs.ie.cowbots.internalActions.experimental2;

import java.awt.Point;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.bdi.brf.BeliefSet;
import edu.udo.cs.ie.cowbots.logic.ELPAtom;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.ui.ICowbotUI;
import edu.udo.cs.ie.cowbots.util.DistanceMap;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Literal;
import jason.asSyntax.NumberTerm;
import jason.asSyntax.Term;

/**
 * 
 * @author dhoelzgen
 *
 */
public class exIsValidScoutDestination extends DefaultInternalAction {

	// map used to represent leader <-> helper lists
	// the key is a leader name, the value is a list of helpers (leader always position 0)
	HashMap<String, List<String>> groups = new HashMap<String, List<String>>();
	
	// map used to represent agent positions
	// the key is an agents name, the value a point
	HashMap<String, Point> agPositions = new HashMap<String, Point>();
	List<String> agHelpers = new LinkedList<String>();
	HashMap<String,Integer> agIndex = new HashMap<String,Integer>();
	HashMap<Integer,String> agInvIndex = new HashMap<Integer,String>();
	
	// the agents name in the mas
	String myself;
	
	// used for logging
	List<String>	debugLog;
	
	/**
	 * Returns true, if the provided target is still a valid scout destination. A
	 * scout destination becomes invalid if one agent of the team reached the target,
	 * or if it turns out that it is impossible to reach.
	 */
	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
	throws Exception {
	
		// Destination position
		int dX = (int) ((NumberTerm) args[0]).solve();
		int dY = (int) ((NumberTerm) args[1]).solve();
		
		ICowbot cowbot = (ICowbot) ts.getAg();
		BeliefSet bs = cowbot.getEpistemicState().getBeliefSet();
		GridMap map = cowbot.getEpistemicState().getMap();
		ICowbotUI ui = cowbot.getUI();
		this.debugLog = new LinkedList<String>();
		
		DistanceMap pathfinder = map.getDistanceMap(false, false);
		
		Point[] aPositions;
		
		// agents name
		myself = ts.getUserAgArch().getAgName();
		
		// build groups and ag Positions
		this.logSeparator("lead: "+myself);
		this.buildGroups(bs);
		
		
		//tv: this makes the helper list dynamic
		if (agInvIndex.size() == 0) {
			System.err.println(this.myself +" exPlanner warning: helper list is empty!");
		}
		aPositions = new Point[agInvIndex.size()];
		
		for (int i = 0; i < agInvIndex.size(); i++) {
			String agKey = agInvIndex.get(i);
			Point agPos =agPositions.get(agKey);
			aPositions[i] = agPos;
		}			
	
		pathfinder.fill(dX, dY);
		
		boolean noOneCanReachTarget = true;
		
		for (int i = 0; i < aPositions.length; i++) {
			Point aPos = aPositions[i];
			LinkedList<Point> agPath = pathfinder.getPath(aPos.x, aPos.y);
			if (agPath == null) return false;
			if (agPath.size() < 6 && agPath.size() > 0) return false;
			if (agPath.size() != 0) noOneCanReachTarget = false; 
		}
		
		if (noOneCanReachTarget) return false;
		
		ui.updateExPlanner(this.debugLog);
		
		return true;
	}
	
	/**
	 * this method sets up some internal data structure for planning,
	 * in this case agent positions and helper lists. the groups list
	 * is based on the beliefs of an agent.
	 * @param bs
	 */
	protected void buildGroups(BeliefSet bs) {
		List<ELPLiteral> leaders = bs.findAll("leader");
		List<ELPLiteral> helpers = bs.findAll("helper");
		List<ELPLiteral> positions = bs.findAll("agentposition");
		Set<String> validHelper = new HashSet<String>();
		
		for (ELPLiteral l : positions) {
			String vh = l.getAtom().getTerm(0);
			if (vh.equalsIgnoreCase("self"))
				vh = myself;
			validHelper.add(vh);
		}
		
		String ldText = "";
				
		agHelpers.clear();
		
		// all leaders
		for (ELPLiteral i : leaders) {
			List<String> j = new LinkedList<String>();
			String ld = i.getAtom().getTerm(0);
			this.groups.put(i.getAtom().getTerm(0), j);
			if (ld.equalsIgnoreCase(this.myself))
				j.add(this.myself);
			ldText += ld+"  "; 
		}
		
		// all helpers with their leader group
		for (ELPLiteral i : helpers) {
			String leadName = i.getAtom().getTerm(0);
			String helpName = i.getAtom().getTerm(1);
			
			List<String> j = this.groups.get(leadName);
			if (j != null)
				j.add(helpName);
		}
		
		
		// prepare output		
		logSeparator("leaders");
		debugLog.add( ldText );		
		logSeparator("helpers");
		for (Entry<String, List<String>> es : groups.entrySet()) {
			String t = es.getKey() + ": ";
			for (String val : es.getValue())
				t += val + " ";
			debugLog.add(t);
		}
		
		// get my helper list
		List<String> myHelpers = groups.get(myself);
		
		// sort that array
		Collections.sort(myHelpers);
		
		// get agent positions		
		logSeparator("my helpers");
		for (ELPLiteral l : positions) {
			ELPAtom atm = l.getAtom();
			String name = atm.getTerm(0);
			if (name.equalsIgnoreCase("self"))
				name = this.myself;
			Point pos = new Point(atm.getTermAsInt(1), atm.getTermAsInt(2)); 
			this.agPositions.put( name, pos ); 			
		}
		
		this.agIndex.clear();
		this.agInvIndex.clear();
		int i = 0;
		for (String ag : myHelpers ) {
			if (validHelper.contains(ag)) {
				this.agIndex.put( ag, i);
				this.agInvIndex.put( i, ag);
				++i;
			}
		}
	
		// now the aPositions can be setup
		for (int j = 0; j < agInvIndex.size(); j++) {
			String agName=agInvIndex.get(j);			
			Point pos = this.agPositions.get( agName );
			if ((agName != null ) && (pos != null))
				debugLog.add(j+": "+agName + " at ("+pos.x+","+pos.y+")");
		}
	}
	
	/**
	 * this method adds a separator (line of "-" to an agents debug log
	 */
	protected void logSeparator(String caption) {
		int sepLen = 20;
		
		if (caption != null) {
			sepLen -= caption.length() -2;
			if (sepLen < 0)
				sepLen = 0;
			String logEntry = "\n-- " + caption + " ";
			for (int i = 0; i < sepLen; i++)
				logEntry += "-";
			debugLog.add(logEntry);
		} else {
			String logEntry = "\n----------------------------";
			debugLog.add(logEntry);
		}
	}
	
}
