package edu.udo.cs.ie.cowbots.internalActions.experimental2;

import java.awt.Point;
import java.util.*;

import org.w3c.dom.Element;

import edu.udo.cs.ie.cowbots.bdi.brf.*;
import edu.udo.cs.ie.cowbots.logic.*;
import edu.udo.cs.ie.cowbots.util.*;


/**
 * a specialized operator for handling map updates.
 * most parts of the map are empty or don't change
 * at all, like obstacles. an inference would reject
 * multiple those cell over and over, so it might
 * be better to handle these things at the belief
 * base level.  
 * 
 * this is the contest optimized version, it performs
 * most updates by itself instead of relying on dlv.
 * 
 * @author Thomas
 *
 */
public class exMapOperator implements IKnowledgeOperator, MapOperator {

	GridMap						map;
	List<ELPLiteral>			newExplorations;
	List<InformationObject>		bels;
	Set<String> 				fenceCoords;
	Set<String>					switchCoords;
	boolean						contestMode;
	
	List<Point>						agentPositions;
	HashMap<String,ELPLiteral>		cowPositions;
	
	public exMapOperator() {
		map = null;
		newExplorations = new LinkedList<ELPLiteral>();
		fenceCoords = new HashSet<String>();
		switchCoords = new HashSet<String>();
		contestMode = true;
	}
		
	@Override
	public void init(Element config) {
		// are we in contest java mode?
		// if so, add gates and fence groups by ourself (the revision
		// will not use any rules at that moment.
		if (config != null) {
			String cm = config.getAttribute("contestmode");
			if (cm != null)
				if (cm.equalsIgnoreCase("true"))
					this.contestMode = true;
		}		
	}

	@Override
	public boolean update(List<InformationObject> updates, BeliefBase bb) {
		
		int cellcount = 0;
		int objcount = 0;
		int explored = 0;
		
		IdentTable it = bb.getIdentifiers();
		List<ELPLiteral> oldMap = new LinkedList<ELPLiteral>();
		bels = bb.getOperatorIO(this);
		
		this.newExplorations.clear();
		
		for(InformationObject io : updates) {
			
			// old content
			ELPLiteral old = null;			
			
			// fetch first literal
			ELPLiteral obj = io.getELP().getRule(0).getHead(0);
			ELPAtom atm = obj.getAtom();
			String functor = atm.getPredicate();
			String objtype = atm.getTerm(0);
			
			if (functor.equalsIgnoreCase("ison")) {
				int x = Integer.parseInt(obj.getAtom().getTerm(1));
				int y = Integer.parseInt(obj.getAtom().getTerm(2));
							
				if ( it.isKindOf(objtype , "cow")) { 														
					// percepted a cow, always add as exploration
					this.newExplorations.add(obj);
					this.newExplorations.add(new ELPAtom("cow",objtype));
					bels.add(io);							
					++objcount;
					
				} else if (it.isKindOf(objtype , "agent")) {

					// percepted an agent
					bels.add(io);
					++objcount;
				} else {
					
					// some kind of obstacle, just put it on the map
					//GridCell<ELPLiteral> gcell = map.getCell(x,y);
					//boolean unexplored gcell.isUnexplored;
					old = map.setContent(x, y, obj);
					//GridCell<ELPLiteral> gcell = map.getCell(x,y);
					
					++cellcount;
					
					
					// but allow switches and fences to be in sight
					// contest: no need to revise that stuff
					if (it.isKindOf(objtype , "fence") ) {
						this.fenceCoords.add( x+"_"+y );						
						//bels.add(io);
					} else if (it.isKindOf(objtype, "switch")) {
						this.switchCoords.add( x+"_"+y );
						//bels.add(io);
					} else	if (it.isKindOf(objtype, "corral")) {
						//bels.add(io);
					}
					
					/*
					if (gcell.isFence)
						this.fenceCoords.add(x+"_"+y);
					else if (gcell.isSwitch)
						this.switchCoords.add(x+"_"+y);
					*/					
					// send newly explored cells
					if (old != null) {
					//if (true) {	
						
						// is the cell no longer unexplored?					
						// fix: transfer elements on corrals to (as some corral cells
						// might be occluded by walls and switches
						if ( it.isKindOf( old.getAtom().getTerm(0), "unexplored") ||
								it.isKindOf( old.getAtom().getTerm(0), "corral")) {
						//if (true) {
							explored++;

							if (it.isKindOf(objtype ,"fence")) {
								this.newExplorations.add( obj );
								this.newExplorations.add( new ELPAtom("fence","fence") );						
								this.fenceCoords.add( obj.getAtom().getTerm(1) +"_" +obj.getAtom().getTerm(2) );
							}
							if (it.isKindOf(objtype , "obstacle")) {
								this.newExplorations.add( obj );
								//this.newExplorations.add( new ELPAtom("obstacle","obstacle") );
							}
							if (it.isKindOf(objtype , "switch")) {
								this.newExplorations.add( obj );
								this.newExplorations.add( new ELPAtom("switch",objtype) );
							}
							if (it.isKindOf(objtype, "empty")) {
								this.newExplorations.add(obj);
								//this.newExplorations.add( new ELPAtom("empty","empty") );
							}
							if (it.isKindOf(objtype, "corral"))
								this.newExplorations.add(obj);
						}
						
					}
				}					
			}

			if (functor.equalsIgnoreCase("isopen")) {
				int x = Integer.parseInt(obj.getAtom().getTerm(0));
				int y = Integer.parseInt(obj.getAtom().getTerm(1));
				
				//bels.add(io);
				map.setFlagNewer(x, y, obj, 3);
				
				// always transfer fence state if we see that fence
				if (this.fenceCoords.contains( ""+x+"_"+y)) {
					this.newExplorations.add(obj);
				}
			}
		}
		
		return false;
	}

	@Override
	public void assignMap(GridMap map) {
		this.map = map;
	}

	
	@Override
	public void deleteFacts(Set<ELPLiteral> toDel) {
		
		if (this.bels == null)
			return;
		
		Iterator<InformationObject> ioIter = this.bels.iterator();
		while (ioIter.hasNext()) {
			InformationObject io = ioIter.next();
			if ( toDel.contains( io.getELP().getRule(0).getHead(0) )) {
				ioIter.remove();
			}
		}	
	}

	@Override
	public List<ELPLiteral> getExploredCells() {
		return this.newExplorations;
	}

	@Override
	public void processBeliefSet(BeliefSet bs, BeliefBase bb) {
		// update revised isopen state here!
		List<ELPLiteral> movers = new LinkedList<ELPLiteral>();
		IdentTable it = bs.getIdentifiers();
		
		for (ELPLiteral l : bs.getBeliefs()) {
			ELPAtom atm = l.getAtom();
			if (atm.getPredicate().equalsIgnoreCase("ison")) {
				boolean isAg = it.isKindOf(atm.getTerm(0), "agent");
				boolean isCow = it.isKindOf(atm.getTerm(0), "cow");
				if (isAg || isCow)
					movers.add(l);
			}
			
			if (atm.getPredicate().equalsIgnoreCase("isopen")) {
				int x = Integer.parseInt(atm.getTerm(0));
				int y = Integer.parseInt(atm.getTerm(1));
				this.map.setFlagNewer(x, y, l,3);
			}
		}
		
		this.map.updateMovers(movers);
		
		// if we are in contest mode, add gates
		Set<String> processed_fences = new HashSet<String>();
		List<Point> fenceCells = new LinkedList<Point>();
		int fenceTimeMax = 0;
		String fenceState = "false";
		
		if (this.contestMode) {
			for (String sc : this.switchCoords) {
				String[] scSplit = sc.split("_");
				int x = Integer.parseInt(scSplit[0]);
				int y = Integer.parseInt(scSplit[1]);
				
				// scan for direction
				int dX = 0;
				int dY = 0;
				if (this.map.isFence(x+1, y))
					dX = 1;
				else if (this.map.isFence(x-1, y))
					dX = -1;
				else if (this.map.isFence(x, y-1))
					dY = -1;
				else if (this.map.isFence(x, y+1))
					dY = 1;
				
				// any direction
				if ((dX != 0) || (dY != 0)) {
					String gatename = "switch_"+x+"_"+y;
					String gatedir = "horizontal";
					if (dY != 0)
						gatedir = "vertical";
					
					bs.addBelief( new ELPAtom("gate", ""+x,""+y,gatename,gatedir) );
					
					x += dX;
					y += dY;
					
					// temp buffer for the gate structure - helps to
					// assign unique fence state to all cells
					fenceCells.clear();
					fenceTimeMax = 0;
					while ( this.map.isFence(x, y)) {
						// get the state of that fence cell
						fenceCells.add( new Point(x,y));
						ELPLiteral isopen = this.map.getFlag(x, y);
						if (isopen != null) {
							int ioTime = isopen.getAtom().getTermAsInt(3);
							if (ioTime > fenceTimeMax) {
								fenceTimeMax = ioTime;
								fenceState = isopen.getAtom().getTerm(2);
							}
						}
						
						bs.addBelief( new ELPAtom("gate", ""+x,""+y,gatename,gatedir) );
						processed_fences.add(x+"_"+y);
						x += dX;
						y += dY;
					}
					
					// overscan fence state
					this.map.setGateState(fenceCells,fenceTimeMax,fenceState);
				}
				
				// scan map for all adjacent fences				
				bs.addBelief( new ELPAtom("contestGate",""+x,""+y));
			}
			
			// find fencegroups
			for (String fc : this.fenceCoords) {
				
				// fence already belongs to a switch
				if (processed_fences.contains( fc ))
					continue;
				
				// ok, fence without a switch, extract coords
				String[] scSplit = fc.split("_");
				int x = Integer.parseInt(scSplit[0]);
				int y = Integer.parseInt(scSplit[1]);
				
				// scan for direction
				int dX = 0;
				int dY = 0;
				if (this.map.isFence(x+1, y))
					dX = 1;
				else if (this.map.isFence(x-1, y))
					dX = -1;
				else if (this.map.isFence(x, y-1))
					dY = -1;
				else if (this.map.isFence(x, y+1))
					dY = 1;
				
				if ((dX != 0) || (dY != 0)) {
					processed_fences.add(fc);
					
					String fgDir = (dX==0)?"vertical":"horizontal";
					// scan forward
					bs.addBelief( new ELPAtom("fencegroup", ""+x,""+y,fgDir) );
					int sX = x;
					int sY = y;
					x+=dX;
					y+=dY;
					while ( this.map.isFence(x, y)) {
						bs.addBelief( new ELPAtom("fencegroup", ""+x,""+y,fgDir) );
						processed_fences.add(x+"_"+y);
						x+=dX;
						y+=dY;						
					}
					dX*=-1;
					dY*=-1;
					x = sX+dX;
					y = sY+dY;
					while ( this.map.isFence(x, y)) {
						bs.addBelief( new ELPAtom("fencegroup", ""+x,""+y,fgDir) );
						processed_fences.add(x+"_"+y);
						x+=dX;
						y+=dY;						
					}
				}
			}
		}
		
		
	}

	@Override
	public ELP getStateProgram() {
		return null;
	}

	@Override
	public GridMap getGridMap() {
		return this.map;
	}

	@Override
	public void reset() {
		if (this.map != null)
			this.map.resetMovers();
		if (bels != null)
			bels.clear();
		newExplorations = new LinkedList<ELPLiteral>();
		fenceCoords = new HashSet<String>();
		switchCoords = new HashSet<String>();
		
		contestMode = true;
	} 
		
			
}
