package edu.udo.cs.ie.cowbots.internalActions.experimental2;

import java.util.*;

import edu.udo.cs.ie.cowbots.bdi.brf.CowbotRevision;
import edu.udo.cs.ie.cowbots.bdi.brf.IdentTable;
import edu.udo.cs.ie.cowbots.bdi.brf.InformationObject;
import edu.udo.cs.ie.cowbots.logic.ELP;
import edu.udo.cs.ie.cowbots.logic.ELPAtom;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.logic.ELPRule;

/**
 * fast version of the cowbot revision class, does
 * not call dlv at all, and can only perform a
 * basic map revision of cow and agent positions
 * and the time predicate. all other revisions
 * or updates are performed in the ex-knowledge
 * operators.
 * 
 * @author Thomas
 *
 */
public class exRevision extends CowbotRevision {

	@Override
	protected void reviseEx() {
		// revise ex is the method that revises the state
		
		// get all information objects
		List<InformationObject> ioList = this.beliefBase.getAllIO();
		
		// some variables used during computation
		Set<ELPLiteral> newBS = new HashSet<ELPLiteral>();
		List<ELPLiteral> ison = new LinkedList<ELPLiteral>();
		List<ELPLiteral> isopen = new LinkedList<ELPLiteral>();
		ELPLiteral time = null;
		Set<ELPLiteral> rejected = new HashSet<ELPLiteral>();
		Set<String> rejectPredicates = new HashSet<String>();
		ELPLiteral myposition = null;
		IdentTable it = this.beliefBase.getIdentifiers();
		
		for (InformationObject io : ioList) {
			ELP e = io.getELP();
			for (ELPRule r : e.getRules() ) {
				// skip rules and constraints and disjunctions
				if ((r.nBody() > 0) || (r.nHead() > 1))
					continue;
				
				// interesting stuff?
				ELPLiteral l = r.getHead(0);
				ELPAtom a = l.getAtom();
				String functor = a.getPredicate();
				
				// process ison on the fly
				if (functor.equals("time")) {
					if (time == null)
						time = l;
					else {
						if (time.getAtom().getTermAsInt(0) > a.getTermAsInt(0)) {
							rejected.add(l);
							rejectPredicates.add("time");
						} else {
							rejectPredicates.add("time");
							rejected.add(time);
							time = l;
						}
					}
				}
				// process ison later
				else if (functor.equals("ison")) {
					if (it.isKindOf(a.getTerm(0), "iam")) {
						myposition = l;						
					}
					ison.add(l);
				}
				// process isopen later
				else if (functor.equals("isopen")) {
					isopen.add(l);
				}
				// ordinary predicate, just add to bs
				else {
					newBS.add(l);
				}
			}			
		}
		
		if (myposition != null)
			newBS.add(myposition);
		
		// now process ison
		HashMap<String, ELPLiteral> cows = new HashMap<String, ELPLiteral>();
		HashMap<String, ELPLiteral> others = new HashMap<String, ELPLiteral>();
		
		// if we didn't received time, there is nothing to do
		if (time != null) {
			int timeInt = time.getAtom().getTermAsInt(0);
			
			rejectPredicates.add("ison");
			rejectPredicates.add("isopen");
			newBS.add(time);
			
			for (ELPLiteral l : ison) {
				String ident = l.getAtom().getTerm(0);
				
				// only accept most recent cow coordinate
				if (it.isKindOf(ident, "cow")) {
					String key = ident;
					ELPLiteral cow = cows.get(key);
					if (cow == null)
						cows.put(key,l);
					else {
						if (l.getAtom().getTermAsInt(3) > cow.getAtom().getTermAsInt(3)) {
							cows.put(key, l);
							rejected.add(cow);
						} else {
							rejected.add(l);						
						}
					}
				}
				
				// drop all old agent positions
				else if (it.isKindOf(ident, "agent")) {
					if (l.getAtom().getTermAsInt(3) < timeInt) {
						
						rejected.add(l);
					} else {
						// drop own position if ident is just ally
						if (myposition != null) {
							if ( getXY(myposition).equals( getXY(l) )) {
								if (l.getAtom().getTerm(0).equals("ally"))
									rejected.add(l);
							} else
								newBS.add(l);
						} else {
							// just add
							newBS.add(l);
						}
					}
				}
				
				// any other ison: keep most recent
				else {
					String key = getXY(l);
					int t = l.getAtom().getTermAsInt(3);
					ELPLiteral k = others.get(key);
					if (k == null) {
						others.put(key,l);
					} else {
						if (t < k.getAtom().getTermAsInt(3))
							rejected.add(l);
						else {
							rejected.add(k);
							others.put(key, l);
						}
					}
				}
			}		
		}
		
		
		
		// add cows, etc, to newbs
		for (ELPLiteral l : cows.values())
			newBS.add(l);
		for (ELPLiteral l : others.values())
			newBS.add(l);
		
		// apply belief changes		
		this.setNewBeliefs(newBS);
		
		// apply rejected predicates
		this.pruneBeliefBase(rejectPredicates, rejected);
	}

	protected String getXY(ELPLiteral l) {
		return l.getAtom().getTerm(1) + "_" + l.getAtom().getTerm(2);
	}
	
}
