package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.util.ArrayList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.Message;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.Literal;
import jason.asSyntax.NumberTerm;
import jason.asSyntax.NumberTermImpl;
import jason.asSyntax.Term;

public class createWaypoints extends DefaultInternalAction {

	public boolean experimental_obstacle = false;
	
	/**
	 * Called to test waypoint mechanism, returns the number
	 * of generated waypoints.
	 * 
	 * CAUTION: Waypoints are not stored anyware at the moment!
	 * Update. Waypoints are populated to BB!
	 */
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
	throws Exception {
		int sX = (int) ((NumberTerm) args[0]).solve();
		int sY = (int) ((NumberTerm) args[1]).solve();
		int tX = (int) ((NumberTerm) args[2]).solve();
		int tY = (int) ((NumberTerm) args[3]).solve();

		List<Literal> waypoints = this.generateWaypoints(ts, sX, sY, tX, tY);
		
		for (Literal lit : waypoints) {
			System.out.println(lit.toString());
			// Add beliefs to Agent's BB
			ts.getAg().addBel(lit);
		}
		un.unifies(args[4], new NumberTermImpl(waypoints.size()));

		return true;

	}
	
	// Wrapper for new function
	public List<Literal> generateWaypoints(TransitionSystem ts,
			int from_x, int from_y, int to_x, int to_y) 
			throws Exception {
		return this.generateWaypoints(ts, from_x, from_y, to_x, to_y, true);
	}

	/**
	 * Generates waypoints according to the cowdriving strategy
	 * 
	 * TODO: Pay attention to the herd's radius
	 * 
	 * @param ts
	 * @param from_x
	 * @param from_y
	 * @param to_x
	 * @param to_y
	 * @param cowdriving true, if waypoints should be generated according to cowdriving strategy
	 * @return
	 * @throws Exception
	 */
	public List<Literal> generateWaypoints(TransitionSystem ts,
			int from_x, int from_y, int to_x, int to_y, boolean cowdriving) 
			throws Exception {

		try {
			astar pathfinder = new astar();
			pathfinder.experimental_obstacle = this.experimental_obstacle;
			
			String[] path = pathfinder.getPath(ts, from_x, from_y, to_x, to_y, false);

			List<Literal> result = new ArrayList<Literal>();

			String lastStep = "initial";
			String lastDirection = "na";
			int stepNo = 0;
			ICowbot cowbot = (ICowbot) ts.getAg();
			GridMap map = cowbot.getEpistemicState().getMap();
			int vX = from_x;
			int vY = from_y;
			int waitForFenceClose = 0;
			
			// TODO: Buggy, if first WP is located on a fence
			Literal possibleFenceWP = Literal.parseLiteral("fixme(waypointonfence)");

			for (String step : path) {
				// System.out.println(step + " (" + map.getContent(vX, vY).toString() + ")" + (step != lastStep ? " CHANGE" : ""));

				// Convention:
				// defaultwp(ag, task, obj, step, x, y, from_direction, to_direction, formationpos, agentcount, time)
				// >> kdefaultwp(step, x, y, from_direction, to_direction)
				
				// Waypoint before fence
				if (map.getContent(vX, vY).getAtom().getTerm(0).contains("fence")) {
					// TODO: Remove last, if it's the same position
					result.add(possibleFenceWP);
					
					lastDirection = possibleFenceWP.getTerm(4).toString();
					stepNo++;
					waitForFenceClose = 1;
				}
				
				if (step != lastStep) {
					Literal newWP;
					if (lastStep == "initial") {
						// First waypoint
						newWP = this.createLiteralWithPredicate("startwp", stepNo, vX, vY, lastDirection, step);
					} else {
						// Default waypoint
						newWP = this.createLiteralWithPredicate("defaultwp", stepNo, vX, vY, lastDirection, step);
					}

					result.add(newWP);

					lastDirection = step;
					stepNo++;
				}
				
				// Waypoint after fence
				// TODO: Bug, if not enough steps after fence
				// TODO: Remove last waypoint, if it's the same position
				if (map.getContent(vX, vY).getAtom().getTerm(0).contains("owncorral") && cowdriving)
					waitForFenceClose = 0;
				if (waitForFenceClose > 0 && !map.getContent(vX, vY).getAtom().getTerm(0).contains("fence"))
					waitForFenceClose++;
				if (waitForFenceClose == 4 && !map.getContent(vX, vY).getAtom().getTerm(0).contains("fence")) {
					waitForFenceClose = 0;
					
					Literal newWP;
					newWP = this.createLiteralWithPredicate("fenceclosewp", stepNo, vX, vY, lastDirection, step);
					result.add(newWP);

					lastDirection = step;
					stepNo++;
				}
					

				possibleFenceWP = this.createLiteralWithPredicate("fenceopenwp", stepNo, vX, vY, lastDirection, step);
				lastStep = step;
				
				if (step.indexOf("north") != -1)
					vY -= 1;
				if (step.indexOf("south") != -1)
					vY += 1;
				if (step.indexOf("east") != -1)
					vX += 1;
				if (step.indexOf("west") != -1)
					vX -= 1;
			}
			
			// Last Waypoint
			result.add(this.createLiteralWithPredicate("endwp", stepNo, vX, vY, lastDirection, "nA"));

			return result;
		} catch (Exception ex) {
			System.err.println("Could not create waypoints: " + ex.toString());
			ex.printStackTrace();
		}

		return null;
	}
	
	/**
	 * Creates Waypoint of type pre, adds prefix k
	 * 
	 * @param pre
	 * @param stepNo
	 * @param vX
	 * @param vY
	 * @param lastDirection
	 * @param step
	 * @return
	 */
	private Literal createLiteralWithPredicate(String pre, int stepNo, int vX, int vY, String lastDirection, String step) {
		return Literal.parseLiteral("kwaypoint(" + pre + "," + String.valueOf(stepNo) + ","
				+ String.valueOf(vX) + "," 
				+ String.valueOf(vY) + "," 
				+ lastDirection + "," + step
				+ ")");
	}
}
