package edu.udo.cs.ie.cowbots.launcher;

import jason.infra.centralised.RunCentralisedMAS;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.Arrays;
import java.util.concurrent.*;
import javax.swing.*;
import java.util.List;

/**
 * wrapper class that starts a multi agent system
 * using jason centralized environment.
 * 
 * @author Thomas
 *
 */
public class CowbotsLauncher  {

	RunCentralisedMAS 	runner;
	
	/**
	 * startup frame shown by the cowbots launcher.
	 * 
	 * @author Thomas
	 *
	 */
	class LaunchFrame extends JFrame implements ActionListener {
		
		CountDownLatch		latch = new CountDownLatch(1);
		JButton 			launch_local;
		JList 				projects;
		List<String[]>		hosts;
		int					returnValue[];
		
		/**
		 * constructor. input parameters are taken
		 * from the local_config.xml
		 * 
		 * @param masProjects	available mas projects
		 * @param hosts			available hosts
		 */
		public LaunchFrame(String masProjects[], List<String[]> hosts) {
			super();
			
			returnValue = new int[2];
			
			JPanel south = new JPanel();
					
			this.hosts = hosts;
			int i = 0;
			for (String[] s : hosts) {
				launch_local = new JButton(s[1]);
				launch_local.addActionListener(this);
				launch_local.setActionCommand(""+i);
				++i;
				south.add( launch_local );				
			}
			this.add(south, BorderLayout.SOUTH);
			
			
			projects = new JList(masProjects);
			projects.setSelectedIndex(0);
			projects.getSelectedIndex();
			this.add(projects, BorderLayout.CENTER);
			
			this.pack();
			this.setTitle("PG545 Cowbots MAS Launcher");
			this.setDefaultCloseOperation(EXIT_ON_CLOSE);
			this.setLocationRelativeTo(null);

			this.setVisible(true);
		}
		
		
		/**
		 * this method blocks a caller unless the
		 * frame is closed, or the user selects
		 * and starts a mas.
		 * 
		 * @return	selected mas information
		 * @throws 	InterruptedException
		 */
		public int[] waitSelect() throws InterruptedException {
			latch.await();
			returnValue[0] = projects.getSelectedIndex();
			return returnValue;
		}



		@Override
		public void actionPerformed(ActionEvent arg0) {
			returnValue[1] = Integer.parseInt(arg0.getActionCommand());
			latch.countDown();
		}
	}
	
	/**
	 * this method starts a mas. if one startup argument
	 * is given, the argument is threated as the mas2j
	 * file to load a mas on localhost. if a second
	 * argument is given, it is always interpreted as
	 * the massim server address the agents should
	 * connect to. if no arguments are given, the
	 * launcher tries to show a gui for mas selection
	 * (otherwise, it exits the application). the
	 * gui is automatically hidden on machines not
	 * capable of displaying a gui, this way the
	 * launcher can be used to start a cowbot mas
	 * on machine without a graphical interface (like
	 * servers, etc).
	 * 
	 * @param args	startup arguments
	 */
	public void launchMAS(String args[]) {
		// create configuration instance here
		CowbotsConfig cfg = CowbotsConfig.getInstance();
		
		// 2nd argument of args is always host override,
		// check if it is avail
		if (args.length >= 2) {
			cfg.setHost(args[1]);
		}
		
		// enumerate all mas2j files
		File f = new File(".");
		File projects[] = f.listFiles( new FileFilter() {
			public boolean accept(File arg0) {
				return arg0.getName().endsWith(".mas2j");
			}
		});
		
		String proj[] = new String[ projects.length ];
		for (int i = 0; i < projects.length; i++) {
			proj[i] = projects[i].getName();
		}
		Arrays.sort(proj);
		
		int selectResult[] = null;
		String startProject = null;
		
		// if args length is 0, no project is given
		// try show mas launcher
		if (args.length == 0)
		{
			try {
				LaunchFrame lf = new LaunchFrame(proj, cfg.getHosts());			
				selectResult = lf.waitSelect();
			} catch (Exception e) {
			}
		}
		
		// no mas project given at all..
		if ((selectResult == null) && (args.length < 1)) {
			System.err.println("error! no mas2j project specified.");
			System.exit(0);
		}
		else if (selectResult != null) {
			startProject = proj[selectResult[0]];
			String sl[] = cfg.getHosts().get(selectResult[1]);
			cfg.setHost( sl[0] );
		} else {
			startProject = args[0];
		}
		
		System.out.println("cowbots mas launching: "+startProject);
		
		System.out.println("configuration:");
		System.out.println("dlv: "+cfg.getDLVPath());
		System.out.println("massim: "+cfg.getMassimPath());
		if (cfg.getHost() != null)
				System.out.println("massim host: "+cfg.getHost());
		
		String[] mas = new String[1]; 
		if (args.length == 0)
			mas[0] = startProject; 
		else
			mas[0] = startProject;
				
		// run mas
		runner = new RunCentralisedMAS();				
		runner.init(mas);
	}
}
