package edu.udo.cs.ie.cowbots.logic.solver;

import edu.udo.cs.ie.cowbots.logic.*;

import java.io.*;
import java.util.*;

/**
 * container class for ELPSource instances
 * 
 * @author Thomas
 *
 */
public class SourceList extends LinkedList<ELPSource> {
	
	public void add(ELP elp) {
		ELPSource src = new ELPSource(elp);
		this.add(src);
	}
	
	public void add(Collection<ELPLiteral> facts) {
		ELPSource src = new ELPSource(facts);
		this.add(src);
	}
	
	public void add(String fileName) {
		ELPSource src = new ELPSource(fileName);
		this.add(src);
	}
	
	
	public void saveAs(String filename) throws IOException {
		BufferedWriter w = new BufferedWriter(new FileWriter(filename));
		
		for (ELPSource src : this) {
			if (src.getType() == ELPSource.SRC_ELP) {
				ELP e = src.resELP;
				ArrayList<ELPRule> rules = e.getRules();
				for ( ELPRule rule : rules) {
					w.write( rule.toString() );
					w.newLine();
				}
			}
			
			if (src.getType() == ELPSource.SRC_FACT) {
				src.addResource(w);
			}
			
			if (src.getType() == ELPSource.SRC_FILE) {
				BufferedReader fr = new BufferedReader( new FileReader(src.getSrcFile()) );
				String ln = fr.readLine();
				while (ln != null){
					w.write(ln);
					w.newLine();
					ln = fr.readLine();
				}  
			}
		}
		
		w.flush();
		w.close();
	}
}
