package edu.udo.cs.ie.cowbots.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import java.util.concurrent.locks.*;

import edu.udo.cs.ie.cowbots.bdi.brf.IdentTable;
import edu.udo.cs.ie.cowbots.bdi.desgen.CDesire;
import edu.udo.cs.ie.cowbots.bdi.desgen.IDesireState;
import edu.udo.cs.ie.cowbots.logic.*;
import edu.udo.cs.ie.cowbots.ui.theme.*;
import edu.udo.cs.ie.cowbots.util.*;
import edu.udo.cs.ie.cowbots.util.cluster.CowCluster;

/**
 * this is a graphical user interface for
 * the cowbot bdi implementation. 
 * 
 * @author Thomas
 *
 */
class BDIViewFrame extends JFrame implements ActionListener {
	
	JTabbedPane					tabs = null;
	JTextArea					beliefState, beliefSet, desireState, intentionState;
	JTextArea					exPlanner;
	MapPanel					worldView;
	JComboBox					agentSelection;
	Map<String, ViewUpdater>	agentList = new HashMap<String, ViewUpdater>();
	String						activeAgent = "";
	ReentrantLock				uiLock = new ReentrantLock();
	
	public BDIViewFrame() {
		super();
		
		initUI();
		this.setTitle("Cowbot BDI Viewer");
		this.setLocation(660, 60);
		this.setSize(400, 400);
		this.pack();
		this.setVisible(true);		
	}
	
	protected void initUI() {
		tabs = new JTabbedPane();
		
		beliefState = new JTextArea();
		JScrollPane sp = new JScrollPane(beliefState);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		tabs.add("BState", sp);
		
		beliefSet = new JTextArea();
		sp = new JScrollPane(beliefSet);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		tabs.add("BSet/other", sp);
		
		worldView = new MapPanel();
		worldView.setSize(400, 400);
		worldView.setPreferredSize( new Dimension(400,400));
		worldView.setVisible(true);
		tabs.add("BSet/ison", worldView);
		
		desireState = new JTextArea();
		sp = new JScrollPane(desireState);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		tabs.add("DesState", sp);
	
		exPlanner = new JTextArea();
		sp = new JScrollPane(exPlanner);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		tabs.add("ExPlanner",sp);
		
		intentionState = new JTextArea();
		sp = new JScrollPane(intentionState);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		tabs.add("IntState", sp);
		
		
		worldView.repaint();
		this.add(tabs);
		
		agentSelection = new JComboBox();
		agentSelection.addActionListener(this);
		this.add(agentSelection, BorderLayout.SOUTH);		
	}
	
	protected void refreshBeliefState(ELP bState) {
		// update belief state display
		this.beliefState.setText(bState.toString());
	}
	
	protected void refreshBeliefSet(Collection<ELPLiteral> bSet) {
		// update belief set
		StringBuilder sb = new StringBuilder();
		ArrayList<DrawCell> db = new ArrayList<DrawCell>();
		CowbotColorTable cct = new CowbotColorTable();
		
		// iterate through belief set, but pass ison to the draw buffer
		Iterator<ELPLiteral> belIter = bSet.iterator();
		while(belIter.hasNext()) {
			ELPLiteral lit = belIter.next();
			ELPAtom atm = lit.getAtom();
			if (atm.getPredicate().equals("ison")) {
				// get coordinates
				int x = Integer.parseInt(atm.getTerm(1));
				int y = Integer.parseInt(atm.getTerm(2));
				int t = Integer.parseInt(atm.getTerm(3));
				
				String t0 = atm.getTerm(0);
				
				if (t0.equals("empty"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_EMPTY)));
				else if (t0.equals("obstacle"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_OBSTACLE)));
				else if (t0.equals("unexplored"))
					db.add(new DrawCell(x,y,Color.DARK_GRAY));
				else if (t0.equals("enemy"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ENEMY)));
				else if (t0.startsWith("cow"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_COW)));
				else if (t0.equals("switch"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_SWITCH)));
				else if (t0.equals("fence"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_FENCE)));
				else if (t0.equals("owncorral"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_CORRALOUR)));
				else if (t0.equals("enemycorral"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_CORRALENEMY)));
				else if (t0.equals(activeAgent))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ACTIVEAGENT)));
				else 
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ALLY)));
				
			} else if (atm.getPredicate().equals("worldsize")){
				int x = Integer.parseInt(atm.getTerm(0));
				int y = Integer.parseInt(atm.getTerm(1));
				worldView.setMapSize(x, y);
				worldView.setAutoScale(true);
			} else {
				sb.append(lit.toString() + "\n");
			}
		}
		this.beliefSet.setText(sb.toString());
		worldView.setDrawBuffer(db);		
		worldView.repaint();
	}
	
	protected void refreshBeliefSet(Collection<ELPLiteral> beliefSet, IdentTable it, GridMap map) {
		StringBuilder sb = new StringBuilder();
		ArrayList<DrawCell> db = new ArrayList<DrawCell>();
		CowbotColorTable cct = new CowbotColorTable();
		LinkedList<IndexedPoint> wpTmp = new LinkedList<IndexedPoint>();
		LinkedList<Point> waypoints = new LinkedList<Point>();
		
		ArrayList<String> myBels = new ArrayList<String>();
		for (ELPLiteral l : beliefSet) {
			ELPAtom atm = l.getAtom();
			
			if (atm.getPredicate().equalsIgnoreCase("ison"))
				continue;
			
			else if (atm.getPredicate().equalsIgnoreCase("waypoint")) {
				waypoints.add( new Point( atm.getTermAsInt(5), atm.getTermAsInt(6) ));
			} else if (atm.getPredicate().equalsIgnoreCase("kwaypoint")) {
				waypoints.add( new Point( atm.getTermAsInt(3), atm.getTermAsInt(4) ));
			} else if (atm.getPredicate().equalsIgnoreCase("wpoint")) {
				waypoints.add( new Point( atm.getTermAsInt(5), atm.getTermAsInt(6) ));
			}
			//sb.append(l.toString()+"\n");
			myBels.add( l.toString() );
		}
		Collections.sort(myBels);
		for (String str : myBels) {
			sb.append(str + "\n");
		}
		this.beliefSet.setText(sb.toString());
		
		
		worldView.setMapSize(map.width(), map.height());
		worldView.setAutoScale(true);
		DrawCell agent = null;
		for (int y = 0; y < map.height(); y++) {
			for (int x = 0; x < map.width(); x++) {
				ELPLiteral ison = map.getContent(x, y);
				if (ison == null)
					continue;
				
				String t0 = ison.getAtom().getTerm(0);
				if (it.isKindOf(t0,"empty" ) )
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_EMPTY)));
				else if (it.isKindOf(t0,"obstacle"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_OBSTACLE)));
				else if (it.isKindOf(t0,"unexplored"))
					db.add(new DrawCell(x,y,Color.DARK_GRAY));
				else if (it.isKindOf(t0,"cow"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_COW)));
				else if (it.isKindOf(t0,"switch"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_SWITCH)));
				else if (it.isKindOf(t0,"fence")) {
					ELPLiteral isopen = map.getFlag(x, y);
					
					if (isopen == null )
						db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_FENCE)));
					else if (isopen.getAtom().getTerm(2).equalsIgnoreCase("false"))
						db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_FENCE)));
					else
						db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_EMPTY)));
				} else if (it.isKindOf(t0,"owncorral"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_CORRALOUR)));
				else if (it.isKindOf(t0,"enemycorral"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_CORRALENEMY)));
				else if (it.isKindOf(t0, "iam"))
					agent = new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ACTIVEAGENT));
				else if (it.isKindOf(t0,"ally"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ALLY)));
				else if (it.isKindOf(t0,"enemy"))
					db.add(new DrawCell(x,y,cct.getColor(CowbotColorTable.C_ENEMY)));
			}
		}

		if (agent != null) {
			db.add(agent);
		}
		
		//this.beliefSet.setText(sb.toString());		
		worldView.setDrawBuffer(db);
		
		// draw waypoints
		worldView.setWaypoints(waypoints);
		
		worldView.repaint();
	}
	
	protected void refreshDesireState(IDesireState desState) {
		// update belief state display
		StringBuilder desString = new StringBuilder();
		
		for (CDesire des : desState) {
			desString.append(des.toString() + "\n");
		}
		
		this.desireState.setText(desString.toString());
	}
	
	protected void refreshIntentionState(String is) {
		this.intentionState.setText(is);
	}
	
	protected void refreshExPlanner(List<String> data) {
		// update belief state display
		StringBuilder desString = new StringBuilder();
		
		for (String s : data) {
			desString.append(s + "\n");
		}
		
		this.exPlanner.setText( desString.toString() );
	}

	/** 
	 * event handler for action events
	 * 
	 * @author Thomas
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		// check for agentSelection events
		if (e.getSource() == agentSelection) {
			setActiveAgent( agentSelection.getSelectedItem().toString() );
		}
	}
	
	/**
	 * sets the bdi frame view to an active agent
	 * 
	 * @param i
	 */
	protected void setActiveAgent(String ag) {
		uiLock.lock();
		this.activeAgent = ag;		
		uiLock.unlock();
	}
	
	public synchronized ViewUpdater registerAgent(String agName) {
		ViewUpdater ret = null;
		
		if (this.agentList.containsKey(agName))
			ret = this.agentList.get(agName);
		else {
			 ret = new ViewUpdater(agName);
			 agentList.put(agName, ret);
			 agentSelection.addItem(agName);
		}
		
		if (agentList.size() == 1) {
			setActiveAgent(agName);
		}
		
		return ret;
	}

	
	
	class ViewUpdater implements ICowbotUI {

		String	agentName;
		
		ViewUpdater(String agentName) {
			this.agentName = agentName;
		}
		
		
		@Override
		public void updateBeliefSet(Collection<ELPLiteral> beliefSet, IdentTable it, GridMap map) {
			uiLock.lock();
			
			if (activeAgent == agentName) {
				if ((it==null) && (map==null))
					refreshBeliefSet(beliefSet);
				else
					refreshBeliefSet(beliefSet,it,map);
			}
			
			uiLock.unlock();
		}

		@Override
		public void updateBeliefState(ELP beliefState) {
			uiLock.lock();
			
			if (activeAgent == agentName) {
				refreshBeliefState(beliefState);
			}
			
			uiLock.unlock();	
		}

		@Override
		public void updateDesireState(IDesireState desState) {
			uiLock.lock();

			if (activeAgent == agentName) {
				refreshDesireState(desState);
			}

			uiLock.unlock();	
		}


		@Override
		public void updateIntentionState(String is) {
			uiLock.lock();

			if (activeAgent == agentName) {
				refreshIntentionState(is);
			}

			uiLock.unlock();	
		}


		@Override
		public void updateCluster(java.util.List<CowCluster> clusters) {
			uiLock.lock();
			
			if (activeAgent == agentName) {
				worldView.setClusterList(clusters);
				worldView.repaint();
			}
			
			uiLock.unlock();
		}


		@Override
		public void updateAgentPath(List<Point> agPath) {
			uiLock.lock();
			
			if (activeAgent == agentName) {
				worldView.setPathPoints(agPath);
				worldView.repaint();
			}
			
			uiLock.unlock();		
		}


		@Override
		public void updateExPlanner(List<String> infos) {
			uiLock.lock();
			
			if (activeAgent == agentName) {
				refreshExPlanner(infos);
			}
			uiLock.unlock();
		}
		
	}
	
	class IndexedPoint {
		public IndexedPoint(Point p, int i) {
			this.p = p;
			this.i = i;
		}
		
		public Point p;
		public int i;
	}
}
