package edu.udo.cs.ie.cowbots.ui;


import java.util.*;
import java.awt.Point;
import java.awt.Rectangle;
import edu.udo.cs.ie.cowbots.bdi.desgen.IDesireState;
import edu.udo.cs.ie.cowbots.logic.*;
import edu.udo.cs.ie.cowbots.bdi.brf.*;
import edu.udo.cs.ie.cowbots.util.*;
import edu.udo.cs.ie.cowbots.util.cluster.CowCluster;

/**
 * basic interface for all bdi components
 * to update their visualization. during
 * initialization, each agent creates
 * his own ui and is responsible for
 * refreshing it.
 *  
 * @author Thomas
 *
 */
public interface ICowbotUI {

	/**
	 * this method is used to update the belief set view
	 * of an agent
	 * 
	 * @param beliefSet
	 * @param it
	 * @param map
	 */
	public void updateBeliefSet(Collection<ELPLiteral> beliefSet, IdentTable it, GridMap map);
	
	/**
	 * this method is used to update a belief state view 
	 * of an agent
	 * @param beliefState
	 */
	public void updateBeliefState(ELP beliefState);
	
	/**
	 * this method is used to update the desire state
	 * view of an agent
	 * 
	 * @param desState
	 */
	public void updateDesireState(IDesireState desState);
	
	/**
	 * this method is used to update the intention
	 * state view of an agent. 
	 * @param is
	 */
	public void updateIntentionState(String is);
	
	/**
	 * this method updates the cow clusters that
	 * an agent created.
	 * 
	 * @param clusters list of rectangles representing cluster rectangle
	 */
	public void updateCluster(List<CowCluster> clusters);
	
	/**
	 * this method updates an agent's path to her desired location
	 * 
	 * @param agPath	agent path in point representation
	 */
	public void updateAgentPath(List<Point> agPath);
	
	
	/**
	 * this method updates an agent's planner status
	 * @param infos
	 */
	public void updateExPlanner(List<String> infos);
}
