package edu.udo.cs.ie.cowbots.util;

import java.awt.Point;

import edu.udo.cs.ie.cowbots.logic.ELPLiteral;

/**
 * 
 * @author dhoelzgen
 *
 */
public class PositionFinder {

	/**
	 * Tries to retrieve the best position, according to the
	 * desired target and the waypoint the target is derived
	 * from.
	 * 
	 * Assumes that wp is a valid position on the map
	 * 
	 * @param wp Waypoint the target is derived from
	 * @param target The target as the agents position in the formation
	 * @param map The map
	 * @return At the best the target, at the worst the waypoint
	 */
	public static Point getBestPositionByWPAndTarget(Point wp, Point target, GridMap map) {
		// Stay inside the map
		if (target.x > map.width()-1) target.x = map.width()-1;
		if (target.y > map.height()-1) target.y = map.height()-1;
		if (target.x < 2) target.x = 2;
		if (target.y < 2) target.y = 2;
		
		Point result = (Point)wp.clone();
		
		// Try to move result starting from waypoint's position as
		// close as posible to the desired target.
		while (!result.equals(target)) {
			// Adjust coordinates in direction of target
			Point candidate = (Point)result.clone();
			if (candidate.x < target.x) candidate.x++;
			if (candidate.x > target.x) candidate.x--;
			if (candidate.y < target.y) candidate.y++;
			if (candidate.y > target.y) candidate.y--;
			
			// If result is a valid position, use it for next iteration
			// otherwise, use last valid position as return value
			ELPLiteral bgnd = map.getBgnd(candidate.x, candidate.y);
			if (bgnd != null) {
				if (bgnd.toString().startsWith("ison(fence")) {
					break;
				}
			}
			// try fence again, might be in foreground (content)
			bgnd=map.getContent(candidate.x, candidate.y);
			if (bgnd.getAtom().getTerm(0).startsWith("fence"))
				break;
			
			if (map.getContent(candidate.x, candidate.y).toString().startsWith("ison(obstacle"))
				break;
			
			result = candidate;
		}
		
		return result;
	}
}
